"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
const aptos_1 = require("aptos");
const child_process_1 = require("child_process");
const fs_1 = __importDefault(require("fs"));
const js_sha3_1 = __importDefault(require("js-sha3"));
const yargs_1 = __importDefault(require("yargs"));
const aptos_2 = require("../aptos");
const consts_2 = require("../consts");
const networks_1 = require("../networks");
const start_validator_1 = require("../start-validator");
const utils_1 = require("../utils");
const README_URL = "https://github.com/wormhole-foundation/wormhole/blob/main/aptos/README.md";
exports.command = "aptos";
exports.desc = "Aptos utilities";
exports.builder = function (y) {
    return (y
        // NOTE: there's no init-nft-bridge, because the native module initialiser
        // functionality has stabilised on mainnet, so we just use that one (which
        // gets called automatically)
        .command("init-token-bridge", "Init token bridge contract", (yargs) => {
        return yargs
            .option("network", consts_2.NETWORK_OPTIONS)
            .option("rpc", consts_2.RPC_OPTIONS);
    }, async (argv) => {
        var _a;
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        const contract_address = (0, utils_1.evm_address)(consts_1.CONTRACTS[network].aptos.token_bridge);
        const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network]["aptos"].rpc;
        await (0, aptos_2.callEntryFunc)(network, rpc, `${contract_address}::token_bridge`, "init", [], []);
    })
        .command("init-wormhole", "Init Wormhole core contract", (yargs) => {
        return yargs
            .option("network", consts_2.NETWORK_OPTIONS)
            .option("rpc", consts_2.RPC_OPTIONS)
            .option("chain-id", {
            describe: "Chain id",
            type: "number",
            default: consts_1.CHAIN_ID_APTOS,
            required: false,
        })
            .option("governance-chain-id", {
            describe: "Governance chain id",
            type: "number",
            default: consts_2.GOVERNANCE_CHAIN,
            required: false,
        })
            .option("governance-address", {
            describe: "Governance address",
            type: "string",
            default: consts_2.GOVERNANCE_EMITTER,
            required: false,
        })
            .option("guardian-address", {
            alias: "g",
            required: true,
            describe: "Initial guardian's addresses (CSV)",
            type: "string",
        });
    }, async (argv) => {
        var _a;
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        const contract_address = (0, utils_1.evm_address)(consts_1.CONTRACTS[network].aptos.core);
        const guardian_addresses = argv["guardian-address"]
            .split(",")
            .map((address) => (0, utils_1.evm_address)(address).substring(24));
        const chain_id = argv["chain-id"];
        const governance_address = (0, utils_1.evm_address)(argv["governance-address"]);
        const governance_chain_id = argv["governance-chain-id"];
        const guardians_serializer = new aptos_1.BCS.Serializer();
        guardians_serializer.serializeU32AsUleb128(guardian_addresses.length);
        guardian_addresses.forEach((address) => guardians_serializer.serializeBytes(Buffer.from(address, "hex")));
        const args = [
            aptos_1.BCS.bcsSerializeUint64(chain_id),
            aptos_1.BCS.bcsSerializeUint64(governance_chain_id),
            aptos_1.BCS.bcsSerializeBytes(Buffer.from(governance_address, "hex")),
            guardians_serializer.getBytes(),
        ];
        const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network]["aptos"].rpc;
        await (0, aptos_2.callEntryFunc)(network, rpc, `${contract_address}::wormhole`, "init", [], args);
    })
        .command("deploy <package-dir>", "Deploy an Aptos package", (yargs) => {
        return yargs
            .positional("package-dir", {
            type: "string",
        })
            .option("network", consts_2.NETWORK_OPTIONS)
            .option("rpc", consts_2.RPC_OPTIONS)
            .option("named-addresses", consts_2.NAMED_ADDRESSES_OPTIONS);
    }, async (argv) => {
        var _a;
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        (0, utils_1.checkBinary)("aptos", README_URL);
        const p = buildPackage(argv["package-dir"], argv["named-addresses"]);
        const b = serializePackage(p);
        const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network]["aptos"].rpc;
        await (0, aptos_2.callEntryFunc)(network, rpc, "0x1::code", "publish_package_txn", [], [b.meta, b.bytecodes]);
        console.log("Deployed:", p.mv_files);
    })
        .command("deploy-resource <seed> <package-dir>", "Deploy an Aptos package using a resource account", (yargs) => {
        return yargs
            .positional("seed", {
            type: "string",
        })
            .positional("package-dir", {
            type: "string",
        })
            .option("network", consts_2.NETWORK_OPTIONS)
            .option("rpc", consts_2.RPC_OPTIONS)
            .option("named-addresses", consts_2.NAMED_ADDRESSES_OPTIONS);
    }, async (argv) => {
        var _a;
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        (0, utils_1.checkBinary)("aptos", README_URL);
        const p = buildPackage(argv["package-dir"], argv["named-addresses"]);
        const b = serializePackage(p);
        const seed = Buffer.from(argv["seed"], "ascii");
        // TODO(csongor): use deployer address from sdk (when it's there)
        let module_name = "0x277fa055b6a73c42c0662d5236c65c864ccbf2d4abd21f174a30c8b786eab84b::deployer";
        if (network == "TESTNET" || network == "MAINNET") {
            module_name =
                "0x0108bc32f7de18a5f6e1e7d6ee7aff9f5fc858d0d87ac0da94dd8d2a5d267d6b::deployer";
        }
        const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network]["aptos"].rpc;
        await (0, aptos_2.callEntryFunc)(network, rpc, module_name, "deploy_derived", [], [b.meta, b.bytecodes, aptos_1.BCS.bcsSerializeBytes(seed)]);
        console.log("Deployed:", p.mv_files);
    })
        .command("send-example-message <message>", "Send example message", (yargs) => {
        return yargs
            .positional("message", {
            type: "string",
        })
            .option("network", consts_2.NETWORK_OPTIONS);
    }, async (argv) => {
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        const rpc = networks_1.NETWORKS[network]["aptos"].rpc;
        // TODO(csongor): use sdk address
        let module_name = "0x277fa055b6a73c42c0662d5236c65c864ccbf2d4abd21f174a30c8b786eab84b::sender";
        if (network == "TESTNET" || network == "MAINNET") {
            module_name =
                "0x0108bc32f7de18a5f6e1e7d6ee7aff9f5fc858d0d87ac0da94dd8d2a5d267d6b::sender";
        }
        await (0, aptos_2.callEntryFunc)(network, rpc, module_name, "send_message", [], [aptos_1.BCS.bcsSerializeBytes(Buffer.from(argv["message"], "ascii"))]);
    })
        .command("derive-resource-account <account> <seed>", "Derive resource account address", (yargs) => {
        return yargs
            .positional("account", {
            type: "string",
        })
            .positional("seed", {
            type: "string",
        });
    }, async (argv) => {
        console.log((0, aptos_2.deriveResourceAccount)(Buffer.from((0, utils_1.hex)(argv["account"]).substring(2), "hex"), argv["seed"]));
    })
        .command("derive-wrapped-address <chain> <origin-address>", "Derive wrapped coin type", (yargs) => {
        return yargs
            .positional("chain", {
            type: "string",
        })
            .positional("origin-address", {
            type: "string",
        })
            .option("network", consts_2.NETWORK_OPTIONS);
    }, async (argv) => {
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        let address = consts_1.CONTRACTS[network].aptos.token_bridge;
        if (address.startsWith("0x"))
            address = address.substring(2);
        const token_bridge_address = Buffer.from(address, "hex");
        (0, consts_1.assertChain)(argv["chain"]);
        const chain = (0, consts_1.coalesceChainId)(argv["chain"]);
        const origin_address = Buffer.from((0, utils_1.evm_address)(argv["origin-address"]), "hex");
        console.log((0, aptos_2.deriveWrappedAssetAddress)(token_bridge_address, chain, origin_address));
    })
        .command("hash-contracts <package-dir>", "Hash contract bytecodes for upgrade", (yargs) => {
        return yargs
            .positional("seed", {
            type: "string",
        })
            .positional("package-dir", {
            type: "string",
        })
            .option("named-addresses", consts_2.NAMED_ADDRESSES_OPTIONS);
    }, (argv) => {
        (0, utils_1.checkBinary)("aptos", README_URL);
        const p = buildPackage(argv["package-dir"], argv["named-addresses"]);
        const b = serializePackage(p);
        console.log(Buffer.from(b.codeHash).toString("hex"));
    })
        .command("upgrade <package-dir>", "Perform upgrade after VAA has been submitted", (_yargs) => {
        return (yargs_1.default
            .positional("package-dir", {
            type: "string",
        })
            // TODO(csongor): once the sdk has the addresses, just look that up
            // based on the module
            .option("contract-address", {
            alias: "a",
            required: true,
            describe: "Address where the wormhole module is deployed",
            type: "string",
        })
            .option("network", consts_2.NETWORK_OPTIONS)
            .option("rpc", consts_2.RPC_OPTIONS)
            .option("named-addresses", consts_2.NAMED_ADDRESSES_OPTIONS));
    }, async (argv) => {
        var _a;
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        (0, utils_1.checkBinary)("aptos", README_URL);
        const p = buildPackage(argv["package-dir"], argv["named-addresses"]);
        const b = serializePackage(p);
        const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network]["aptos"].rpc;
        // TODO(csongor): use deployer address from sdk (when it's there)
        const hash = await (0, aptos_2.callEntryFunc)(network, rpc, `${argv["contract-address"]}::contract_upgrade`, "upgrade", [], [b.meta, b.bytecodes]);
        console.log("Deployed:", p.mv_files);
        console.log(hash);
    })
        .command("migrate", "Perform migration after contract upgrade", (_yargs) => {
        return (yargs_1.default
            // TODO(csongor): once the sdk has the addresses, just look that up
            // based on the module
            .option("contract-address", {
            alias: "a",
            required: true,
            describe: "Address where the wormhole module is deployed",
            type: "string",
        })
            .option("network", consts_2.NETWORK_OPTIONS)
            .option("rpc", consts_2.RPC_OPTIONS));
    }, async (argv) => {
        var _a;
        const network = argv.network.toUpperCase();
        (0, utils_1.assertNetwork)(network);
        (0, utils_1.checkBinary)("aptos", README_URL);
        const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network]["aptos"].rpc;
        // TODO(csongor): use deployer address from sdk (when it's there)
        const hash = await (0, aptos_2.callEntryFunc)(network, rpc, `${argv["contract-address"]}::contract_upgrade`, "migrate", [], []);
        console.log(hash);
    })
        // TODO - make faucet support testnet in additional to localnet
        .command("faucet", "Request money from the faucet for a given account", (yargs) => {
        return yargs
            .option("rpc", consts_2.RPC_OPTIONS)
            .option("faucet", {
            alias: "f",
            required: false,
            describe: "Faucet url",
            type: "string",
        })
            .option("amount", {
            alias: "m",
            required: false,
            describe: "Amount to request",
            type: "number",
        })
            .option("account", {
            alias: "a",
            required: false,
            describe: "Account to fund",
            type: "string",
        });
    }, async (argv) => {
        let NODE_URL = "http://0.0.0.0:8080/v1";
        let FAUCET_URL = "http://0.0.0.0:8081";
        let account = "0x277fa055b6a73c42c0662d5236c65c864ccbf2d4abd21f174a30c8b786eab84b";
        let amount = 40000000;
        if (argv.faucet != undefined) {
            FAUCET_URL = argv.faucet;
        }
        if (argv.rpc != undefined) {
            NODE_URL = argv.rpc;
        }
        if (argv.amount != undefined) {
            amount = argv.amount;
        }
        if (argv.account != undefined) {
            account = argv.account;
        }
        const faucetClient = new aptos_1.FaucetClient(NODE_URL, FAUCET_URL);
        await faucetClient.fundAccount(account, amount);
        console.log(`Funded ${account} with ${amount} coins`);
    })
        .command("start-validator", "Start a local aptos validator", (yargs) => {
        return yargs.option("validator-args", start_validator_1.validator_args);
    }, (argv) => {
        (0, utils_1.checkBinary)("aptos", README_URL);
        const os = require("os");
        const dir = os.homedir();
        const cmd = `cd ${dir} && aptos node run-local-testnet --with-faucet --force-restart --assume-yes`;
        (0, start_validator_1.runCommand)(cmd, argv["validator-args"]);
    })
        .strict()
        .demandCommand());
};
function buildPackage(dir, addrs) {
    const named_addresses = addrs ? ["--named-addresses", addrs] : [];
    const aptos = (0, child_process_1.spawnSync)("aptos", [
        "move",
        "compile",
        "--save-metadata",
        "--included-artifacts",
        "none",
        "--package-dir",
        dir,
        ...named_addresses,
    ]);
    if (aptos.status !== 0) {
        console.error(aptos.stderr.toString("utf8"));
        console.error(aptos.stdout.toString("utf8"));
        process.exit(1);
    }
    const result = JSON.parse(aptos.stdout.toString("utf8"));
    const buildDirs = fs_1.default
        .readdirSync(`${dir}/build`, { withFileTypes: true })
        .filter((dirent) => dirent.isDirectory())
        .map((dirent) => dirent.name);
    if (buildDirs.length !== 1) {
        console.error(`Unexpected directory structure in ${dir}/build: expected a single directory`);
        process.exit(1);
    }
    const buildDir = `${dir}/build/${buildDirs[0]}`;
    return {
        meta_file: `${buildDir}/package-metadata.bcs`,
        mv_files: result["Result"].map((mod) => `${buildDir}/bytecode_modules/${mod.split("::")[1]}.mv`),
    };
}
function serializePackage(p) {
    const metaBytes = fs_1.default.readFileSync(p.meta_file);
    const packageMetadataSerializer = new aptos_1.BCS.Serializer();
    packageMetadataSerializer.serializeBytes(metaBytes);
    const serializedPackageMetadata = packageMetadataSerializer.getBytes();
    const modules = p.mv_files.map((file) => fs_1.default.readFileSync(file));
    const serializer = new aptos_1.BCS.Serializer();
    serializer.serializeU32AsUleb128(modules.length);
    modules.forEach((module) => serializer.serializeBytes(module));
    const serializedModules = serializer.getBytes();
    const hashes = [metaBytes]
        .concat(modules)
        .map((x) => Buffer.from(js_sha3_1.default.keccak256(x), "hex"));
    const codeHash = Buffer.from(js_sha3_1.default.keccak256(Buffer.concat(hashes)), "hex");
    return {
        meta: serializedPackageMetadata,
        bytecodes: serializedModules,
        codeHash,
    };
}
