"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
const encoding_1 = require("@cosmjs/encoding");
const bs58_1 = __importDefault(require("bs58"));
const js_sha3_1 = require("js-sha3");
const consts_2 = require("../consts");
const utils_1 = require("../utils");
const vaa_1 = require("../vaa");
function makeVAA(emitterChain, emitterAddress, signers, p) {
    let v = {
        version: 1,
        guardianSetIndex: 0,
        signatures: [],
        timestamp: 1,
        nonce: 1,
        emitterChain: emitterChain,
        emitterAddress: emitterAddress,
        sequence: BigInt(Math.floor(Math.random() * 100000000)),
        consistencyLevel: 0,
        payload: p,
    };
    v.signatures = (0, vaa_1.sign)(signers, v);
    return v;
}
exports.command = "generate";
exports.desc = "generate VAAs (devnet and testnet only)";
exports.builder = function (y) {
    return (y
        .option("guardian-secret", {
        alias: "g",
        required: true,
        describe: "Guardians' secret keys (CSV)",
        type: "string",
    })
        // Registration
        .command("registration", "Generate registration VAA", (yargs) => {
        return yargs
            .option("chain", {
            alias: "c",
            describe: "Chain to register",
            type: "string",
            choices: Object.keys(consts_1.CHAINS),
            required: true,
        })
            .option("contract-address", {
            alias: "a",
            describe: "Contract to register",
            type: "string",
            required: true,
        })
            .option("module", {
            alias: "m",
            describe: "Module to upgrade",
            type: "string",
            choices: ["NFTBridge", "TokenBridge"],
            required: true,
        });
    }, (argv) => {
        let module = argv["module"];
        (0, consts_1.assertChain)(argv["chain"]);
        let payload = {
            module,
            type: "RegisterChain",
            chain: 0,
            emitterChain: (0, consts_1.toChainId)(argv["chain"]),
            emitterAddress: parseAddress(argv["chain"], argv["contract-address"]),
        };
        let v = makeVAA(consts_2.GOVERNANCE_CHAIN, consts_2.GOVERNANCE_EMITTER, argv["guardian-secret"].split(","), payload);
        console.log((0, vaa_1.serialiseVAA)(v));
    })
        // Upgrade
        .command("upgrade", "Generate contract upgrade VAA", (yargs) => {
        return yargs
            .option("chain", {
            alias: "c",
            describe: "Chain to upgrade",
            type: "string",
            choices: Object.keys(consts_1.CHAINS),
            required: true,
        })
            .option("contract-address", {
            alias: "a",
            describe: "Contract to upgrade to",
            type: "string",
            required: true,
        })
            .option("module", {
            alias: "m",
            describe: "Module to upgrade",
            type: "string",
            choices: ["Core", "NFTBridge", "TokenBridge"],
            required: true,
        });
    }, (argv) => {
        (0, consts_1.assertChain)(argv["chain"]);
        let module = argv["module"];
        let payload = {
            module,
            type: "ContractUpgrade",
            chain: (0, consts_1.toChainId)(argv["chain"]),
            address: parseCodeAddress(argv["chain"], argv["contract-address"]),
        };
        let v = makeVAA(consts_2.GOVERNANCE_CHAIN, consts_2.GOVERNANCE_EMITTER, argv["guardian-secret"].split(","), payload);
        console.log((0, vaa_1.serialiseVAA)(v));
    })
        .command("attestation", "Generate a token attestation VAA", (yargs) => {
        return yargs
            .option("emitter-chain", {
            alias: "e",
            describe: "Emitter chain of the VAA",
            type: "string",
            choices: Object.keys(consts_1.CHAINS),
            required: true,
        })
            .option("emitter-address", {
            alias: "f",
            describe: "Emitter address of the VAA",
            type: "string",
            required: true,
        })
            .option("chain", {
            alias: "c",
            describe: "Token's chain",
            type: "string",
            choices: Object.keys(consts_1.CHAINS),
            required: true,
        })
            .option("token-address", {
            alias: "a",
            describe: "Token's address",
            type: "string",
            required: true,
        })
            .option("decimals", {
            alias: "d",
            describe: "Token's decimals",
            type: "number",
            required: true,
        })
            .option("symbol", {
            alias: "s",
            describe: "Token's symbol",
            type: "string",
            required: true,
        })
            .option("name", {
            alias: "n",
            describe: "Token's name",
            type: "string",
            required: true,
        });
    }, (argv) => {
        let emitter_chain = argv["emitter-chain"];
        (0, consts_1.assertChain)(argv["chain"]);
        (0, consts_1.assertChain)(emitter_chain);
        let payload = {
            module: "TokenBridge",
            type: "AttestMeta",
            chain: 0,
            tokenAddress: parseAddress(argv["chain"], argv["token-address"]),
            tokenChain: (0, consts_1.toChainId)(argv["chain"]),
            decimals: argv["decimals"],
            symbol: argv["symbol"],
            name: argv["name"],
        };
        let v = makeVAA((0, consts_1.toChainId)(emitter_chain), parseAddress(emitter_chain, argv["emitter-address"]), argv["guardian-secret"].split(","), payload);
        console.log((0, vaa_1.serialiseVAA)(v));
    })
        // RecoverChainId
        .command("recover-chain-id", "Generate a recover chain ID VAA", (yargs) => {
        return yargs
            .option("module", {
            alias: "m",
            describe: "Module to upgrade",
            type: "string",
            choices: ["Core", "NFTBridge", "TokenBridge"],
            required: true,
        })
            .option("evm-chain-id", {
            alias: "e",
            describe: "EVM chain ID to set",
            type: "string",
            required: true,
        })
            .option("new-chain-id", {
            alias: "c",
            describe: "New chain ID to set",
            type: "number",
            required: true,
        });
    }, (argv) => {
        let module = argv["module"];
        let payload = {
            module,
            type: "RecoverChainId",
            evmChainId: BigInt(argv["evm-chain-id"]),
            newChainId: argv["new-chain-id"],
        };
        let v = makeVAA(consts_2.GOVERNANCE_CHAIN, consts_2.GOVERNANCE_EMITTER, argv["guardian-secret"].split(","), payload);
        console.log((0, vaa_1.serialiseVAA)(v));
    }));
};
function parseAddress(chain, address) {
    if (chain === "unset") {
        throw Error("Chain unset");
    }
    else if ((0, consts_1.isEVMChain)(chain)) {
        return "0x" + (0, utils_1.evm_address)(address);
    }
    else if ((0, consts_1.isCosmWasmChain)(chain)) {
        return "0x" + (0, encoding_1.toHex)((0, encoding_1.fromBech32)(address).data).padStart(64, "0");
    }
    else if (chain === "solana" || chain === "pythnet") {
        return "0x" + (0, encoding_1.toHex)(bs58_1.default.decode(address)).padStart(64, "0");
    }
    else if (chain === "algorand") {
        // TODO: is there a better native format for algorand?
        return "0x" + (0, utils_1.evm_address)(address);
    }
    else if (chain === "near") {
        return "0x" + (0, utils_1.evm_address)(address);
    }
    else if (chain === "osmosis") {
        throw Error("OSMOSIS is not supported yet");
    }
    else if (chain === "sui") {
        return "0x" + (0, utils_1.evm_address)(address);
    }
    else if (chain === "aptos") {
        if (/^(0x)?[0-9a-fA-F]+$/.test(address)) {
            return "0x" + (0, utils_1.evm_address)(address);
        }
        return (0, js_sha3_1.sha3_256)(Buffer.from(address)); // address is hash of fully qualified type
    }
    else if (chain === "wormchain") {
        const sdk = require("@certusone/wormhole-sdk/lib/cjs/utils/array");
        return "0x" + sdk.tryNativeToHexString(address, chain);
    }
    else if (chain === "btc") {
        throw Error("btc is not supported yet");
    }
    else {
        (0, vaa_1.impossible)(chain);
    }
}
function parseCodeAddress(chain, address) {
    if ((0, consts_1.isCosmWasmChain)(chain)) {
        return "0x" + parseInt(address, 10).toString(16).padStart(64, "0");
    }
    else {
        return parseAddress(chain, address);
    }
}
