"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addSetupCommands = void 0;
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
const tokenBridge_1 = require("@certusone/wormhole-sdk/lib/cjs/vaa/tokenBridge");
const sui_js_1 = require("@mysten/sui.js");
const dotenv_1 = __importDefault(require("dotenv"));
const fs_1 = __importDefault(require("fs"));
const consts_2 = require("../../consts");
const networks_1 = require("../../networks");
const sui_1 = require("../../sui");
const deploy_1 = require("./deploy");
const init_1 = require("./init");
const addSetupCommands = (y) => y.command("setup-devnet", "Setup devnet by deploying and initializing core and token bridges and submitting chain registrations.", (yargs) => {
    return yargs
        .option("private-key", {
        alias: "k",
        describe: "Custom private key to sign txs",
        required: false,
        type: "string",
    })
        .option("rpc", consts_2.RPC_OPTIONS);
}, async (argv) => {
    var _a;
    const network = "DEVNET";
    const privateKey = argv["private-key"];
    const rpc = (_a = argv.rpc) !== null && _a !== void 0 ? _a : networks_1.NETWORKS[network].sui.rpc;
    // Deploy core bridge
    console.log("[1/4] Deploying core bridge...");
    const coreBridgeDeployRes = await (0, deploy_1.deploy)(network, "wormhole", rpc, privateKey);
    (0, sui_1.assertSuccess)(coreBridgeDeployRes, "Core bridge deployment failed.");
    (0, sui_1.logTransactionDigest)(coreBridgeDeployRes);
    (0, sui_1.logPublishedPackageId)(coreBridgeDeployRes);
    // Init core bridge
    console.log("\n[2/4] Initializing core bridge...");
    const coreBridgePackageId = (0, sui_1.getPublishedPackageId)(coreBridgeDeployRes);
    const coreBridgeInitRes = await (0, init_1.initWormhole)(network, coreBridgePackageId, consts_2.INITIAL_GUARDIAN_DEVNET, consts_2.GOVERNANCE_CHAIN, 0, consts_2.GOVERNANCE_EMITTER, rpc, privateKey);
    const coreBridgeStateObjectId = (0, sui_1.getCreatedObjects)(coreBridgeInitRes).find((e) => (0, sui_1.isSameType)(e.type, `${coreBridgePackageId}::state::State`)).objectId;
    (0, sui_1.assertSuccess)(coreBridgeInitRes, "Core bridge initialization failed.");
    (0, sui_1.logTransactionDigest)(coreBridgeInitRes);
    console.log("Core bridge state object ID", coreBridgeStateObjectId);
    // Deploy token bridge
    console.log("\n[3/4] Deploying token bridge...");
    const tokenBridgeDeployRes = await (0, deploy_1.deploy)(network, "token_bridge", rpc, privateKey);
    (0, sui_1.assertSuccess)(tokenBridgeDeployRes, "Token bridge deployment failed.");
    (0, sui_1.logTransactionDigest)(tokenBridgeDeployRes);
    (0, sui_1.logPublishedPackageId)(tokenBridgeDeployRes);
    // Init token bridge
    console.log("\n[4/4] Initializing token bridge...");
    const tokenBridgePackageId = (0, sui_1.getPublishedPackageId)(tokenBridgeDeployRes);
    const tokenBridgeInitRes = await (0, init_1.initTokenBridge)(network, tokenBridgePackageId, coreBridgeStateObjectId, consts_2.GOVERNANCE_CHAIN, consts_2.GOVERNANCE_EMITTER, rpc, privateKey);
    const tokenBridgeStateObjectId = (0, sui_1.getCreatedObjects)(tokenBridgeInitRes).find((e) => (0, sui_1.isSameType)(e.type, `${tokenBridgePackageId}::state::State`)).objectId;
    (0, sui_1.assertSuccess)(tokenBridgeInitRes, "Token bridge initialization failed.");
    (0, sui_1.logTransactionDigest)(tokenBridgeInitRes);
    console.log("Token bridge state object ID", tokenBridgeStateObjectId);
    // Deploy example app
    console.log("\n[+1/3] Deploying example app...");
    const exampleAppDeployRes = await (0, deploy_1.deploy)(network, "examples/core_messages", rpc, privateKey);
    (0, sui_1.logTransactionDigest)(tokenBridgeDeployRes);
    (0, sui_1.logPublishedPackageId)(tokenBridgeDeployRes);
    // Init example app
    console.log("\n[+2/3] Initializing example app...");
    const exampleAppPackageId = (0, sui_1.getPublishedPackageId)(exampleAppDeployRes);
    const exampleAppInitRes = await (0, init_1.initExampleApp)(network, exampleAppPackageId, coreBridgeStateObjectId, rpc, privateKey);
    const exampleAppStateObjectId = (0, sui_1.getCreatedObjects)(exampleAppInitRes).find((e) => (0, sui_1.isSameType)(e.type, `${exampleAppPackageId}::sender::State`)).objectId;
    (0, sui_1.logTransactionDigest)(exampleAppInitRes);
    console.log("Example app state object ID", exampleAppStateObjectId);
    // Deploy example coins
    console.log("\n[+3/3] Deploying example coins...");
    const coinsDeployRes = await (0, deploy_1.deploy)(network, "examples/coins", rpc, privateKey);
    (0, sui_1.logTransactionDigest)(coinsDeployRes);
    (0, sui_1.logPublishedPackageId)(coinsDeployRes);
    // Print publish message command for convenience
    let publishCommand = `\nPublish message command: worm sui publish-example-message -n devnet -p "${exampleAppPackageId}" -s "${exampleAppStateObjectId}" -w "${coreBridgeStateObjectId}" -m "hello"`;
    if (argv.rpc)
        publishCommand += ` -r "${argv.rpc}"`;
    if (privateKey)
        publishCommand += ` -k "${privateKey}"`;
    console.log(publishCommand);
    // Dump summary
    const provider = (0, sui_1.getProvider)(network, rpc);
    const emitterCapObjectId = await getEmitterCapObjectId(provider, tokenBridgeStateObjectId);
    console.log("\nSummary:");
    console.log("  Core bridge package ID", coreBridgePackageId);
    console.log("  Core bridge state object ID", coreBridgeStateObjectId);
    console.log("  Token bridge package ID", tokenBridgePackageId);
    console.log("  Token bridge state object ID", tokenBridgeStateObjectId);
    console.log("  Token bridge emitter cap ID", emitterCapObjectId);
    // Chain registrations
    console.log("\nChain registrations:");
    const envPath = `${process.cwd()}/.env`;
    if (!fs_1.default.existsSync(envPath)) {
        throw new Error(`Couldn't find .env file at ${envPath}.`);
    }
    dotenv_1.default.config({ path: envPath });
    const signer = (0, sui_1.getSigner)(provider, network, privateKey);
    const tx = new sui_js_1.TransactionBlock();
    tx.setGasBudget(10000000000);
    const registrations = [];
    for (const key in process.env) {
        if (/^REGISTER_(.+)_TOKEN_BRIDGE_VAA$/.test(key)) {
            // Get VAA info
            const vaa = Buffer.from(String(process.env[key]), "hex");
            const { foreignChain, module } = (0, tokenBridge_1.parseTokenBridgeRegisterChainVaa)(vaa);
            const chain = (0, consts_1.coalesceChainName)(foreignChain);
            registrations.push({ chain, module });
            // Register
            await (0, sui_1.registerChain)(provider, network, vaa, coreBridgeStateObjectId, tokenBridgeStateObjectId, tx);
        }
    }
    const registerRes = await (0, sui_1.executeTransactionBlock)(signer, tx);
    (0, sui_1.assertSuccess)(registerRes, "Chain registrations failed.");
    // Log registered bridges
    for (const registration of registrations) {
        console.log(`  ${registration.chain} ${registration.module}... done`);
    }
    console.log("Transaction digest:", (0, sui_js_1.getTransactionDigest)(registerRes));
    // Done!
    console.log("\nDone!");
});
exports.addSetupCommands = addSetupCommands;
const getEmitterCapObjectId = async (provider, tokenBridgeStateObjectId) => {
    return (0, sui_js_1.getObjectFields)(await provider.getObject({
        id: tokenBridgeStateObjectId,
        options: {
            showContent: true,
        },
    })).emitter_cap.fields.id.id;
};
