"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute_injective = void 0;
const consts_1 = require("@certusone/wormhole-sdk/lib/cjs/utils/consts");
const networks_1 = require("@injectivelabs/networks");
const sdk_ts_1 = require("@injectivelabs/sdk-ts");
const utils_1 = require("@injectivelabs/utils");
const js_base64_1 = require("js-base64");
const networks_2 = require("./networks");
const vaa_1 = require("./vaa");
async function execute_injective(payload, vaa, environment) {
    if (environment === "DEVNET") {
        throw new Error("Injective is not supported in DEVNET");
    }
    const chainName = "injective";
    let n = networks_2.NETWORKS[environment][chainName];
    if (!n.key) {
        throw Error(`No ${environment} key defined for Injective`);
    }
    let contracts = consts_1.CONTRACTS[environment][chainName];
    const endPoint = environment === "MAINNET" ? networks_1.Network.MainnetK8s : networks_1.Network.TestnetK8s;
    const network = (0, networks_1.getNetworkInfo)(endPoint);
    const walletPKHash = n.key;
    const walletPK = sdk_ts_1.PrivateKey.fromMnemonic(walletPKHash);
    const walletInjAddr = walletPK.toBech32();
    const walletPublicKey = walletPK.toPublicKey().toBase64();
    let target_contract;
    let action;
    let execute_msg;
    switch (payload.module) {
        case "Core":
            target_contract = contracts.core;
            action = "submit_v_a_a";
            execute_msg = {
                [action]: {
                    vaa: (0, js_base64_1.fromUint8Array)(vaa),
                },
            };
            switch (payload.type) {
                case "GuardianSetUpgrade":
                    console.log("Submitting new guardian set");
                    break;
                case "ContractUpgrade":
                    console.log("Upgrading core contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on injective");
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "NFTBridge":
            if (contracts.nft_bridge === undefined) {
                // NOTE: this code can safely be removed once the injective NFT bridge is
                // released, but it's fine for it to stay, as the condition will just be
                // skipped once 'contracts.nft_bridge' is defined
                throw new Error("NFT bridge not supported yet for injective");
            }
            target_contract = contracts.nft_bridge;
            action = "submit_vaa";
            execute_msg = {
                [action]: {
                    data: (0, js_base64_1.fromUint8Array)(vaa),
                },
            };
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on injective");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                default:
                    (0, vaa_1.impossible)(payload);
            }
            break;
        case "TokenBridge":
            console.log("contracts:", contracts);
            if (contracts.token_bridge === undefined) {
                throw new Error("contracts.token_bridge is undefined");
            }
            target_contract = contracts.token_bridge;
            action = "submit_vaa";
            execute_msg = {
                [action]: {
                    data: (0, js_base64_1.fromUint8Array)(vaa),
                },
            };
            switch (payload.type) {
                case "ContractUpgrade":
                    console.log("Upgrading contract");
                    break;
                case "RecoverChainId":
                    throw new Error("RecoverChainId not supported on injective");
                case "RegisterChain":
                    console.log("Registering chain");
                    break;
                case "Transfer":
                    console.log("Completing transfer");
                    break;
                case "AttestMeta":
                    console.log("Creating wrapped token");
                    break;
                case "TransferWithPayload":
                    throw Error("Can't complete payload 3 transfer from CLI");
                default:
                    (0, vaa_1.impossible)(payload);
                    break;
            }
            break;
        default:
            target_contract = (0, vaa_1.impossible)(payload);
            execute_msg = (0, vaa_1.impossible)(payload);
    }
    console.log("execute_msg", execute_msg);
    const transaction = sdk_ts_1.MsgExecuteContractCompat.fromJSON({
        sender: walletInjAddr,
        contractAddress: target_contract,
        exec: {
            action,
            msg: {
                ...execute_msg[action],
            },
        },
    });
    console.log("transaction:", transaction);
    const accountDetails = await new sdk_ts_1.ChainRestAuthApi(network.rest).fetchAccount(walletInjAddr);
    const { signBytes, txRaw } = (0, sdk_ts_1.createTransaction)({
        message: transaction,
        memo: "",
        fee: (0, utils_1.getStdFee)((parseInt(utils_1.DEFAULT_STD_FEE.gas, 10) * 2.5).toString()),
        pubKey: walletPublicKey,
        sequence: parseInt(accountDetails.account.base_account.sequence, 10),
        accountNumber: parseInt(accountDetails.account.base_account.account_number, 10),
        chainId: network.chainId,
    });
    console.log("txRaw", txRaw);
    console.log("sign transaction...");
    /** Sign transaction */
    const sig = await walletPK.sign(Buffer.from(signBytes));
    /** Append Signatures */
    txRaw.signatures = [sig];
    const txService = new sdk_ts_1.TxGrpcApi(network.grpc);
    console.log("simulate transaction...");
    /** Simulate transaction */
    try {
        const simulationResponse = await txService.simulate(txRaw);
        console.log(`Transaction simulation response: ${JSON.stringify(simulationResponse.gasInfo)}`);
    }
    catch (e) {
        console.log("Failed to simulate:", e);
        return;
    }
    console.log("broadcast transaction...");
    /** Broadcast transaction */
    const txResponse = await txService.broadcast(txRaw);
    console.log("txResponse", txResponse);
    if (txResponse.code !== 0) {
        console.log(`Transaction failed: ${txResponse.rawLog}`);
    }
    else {
        console.log(`Broadcasted transaction hash: ${JSON.stringify(txResponse.txHash)}`);
    }
}
exports.execute_injective = execute_injective;
