"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildCoin = void 0;
const fs_1 = __importDefault(require("fs"));
const MoveToml_1 = require("./MoveToml");
const publish_1 = require("./publish");
const utils_1 = require("./utils");
const buildCoin = async (provider, network, packagePath, coreBridgeStateObjectId, tokenBridgeStateObjectId, version, decimals) => {
    const coreBridgePackageId = await (0, utils_1.getPackageId)(provider, coreBridgeStateObjectId);
    const tokenBridgePackageId = await (0, utils_1.getPackageId)(provider, tokenBridgeStateObjectId);
    try {
        setupCoin(network, packagePath, coreBridgePackageId, tokenBridgePackageId, version, decimals);
        return (0, publish_1.buildPackage)(`${packagePath}/wrapped_coin`);
    }
    finally {
        cleanupCoin(`${packagePath}/wrapped_coin`);
    }
};
exports.buildCoin = buildCoin;
const setupCoin = (network, packagePath, coreBridgePackageId, tokenBridgePackageId, version, decimals) => {
    // Check to see if the given version string is valid. We don't include the
    // end boundary in the regex to accomodate versions such as V__0_1_0_patch,
    // in the off chance we need such a naming scheme.
    if (!/^V__[0-9]+_[0-9]+_[0-9]+/.test(version)) {
        throw new Error(`Invalid version ${version}`);
    }
    // Assemble package directory
    fs_1.default.rmSync(`${packagePath}/wrapped_coin`, { recursive: true, force: true });
    fs_1.default.mkdirSync(`${packagePath}/wrapped_coin/sources`, { recursive: true });
    // Replace template variables
    const coinTemplate = fs_1.default
        .readFileSync(`${packagePath}/templates/wrapped_coin/sources/coin.move`, "utf8")
        .toString();
    const coin = coinTemplate
        .replace(/{{DECIMALS}}/, decimals.toString())
        .replace(/{{VERSION}}/g, version);
    fs_1.default.writeFileSync(`${packagePath}/wrapped_coin/sources/coin.move`, coin, "utf8");
    // Substitute dependency package IDs
    const toml = new MoveToml_1.MoveToml(`${packagePath}/templates/wrapped_coin/Move.toml`)
        .updateRow("addresses", "wormhole", coreBridgePackageId)
        .updateRow("addresses", "token_bridge", tokenBridgePackageId)
        .serialize();
    const tomlPath = `${packagePath}/wrapped_coin/Move.toml`;
    fs_1.default.writeFileSync(tomlPath, toml, "utf8");
    // Setup dependencies
    const paths = (0, publish_1.getAllLocalPackageDependencyPaths)(tomlPath);
    for (const dependencyPath of paths) {
        // todo(aki): the 4th param is a hack that makes this work, but doesn't
        // necessarily make sense. We should probably revisit this later.
        (0, publish_1.setupMainToml)(dependencyPath, network, false, network !== "DEVNET");
        if (network === "DEVNET") {
            const dependencyToml = new MoveToml_1.MoveToml((0, publish_1.getDefaultTomlPath)(dependencyPath));
            switch ((0, publish_1.getPackageNameFromPath)(dependencyPath)) {
                case "wormhole":
                    dependencyToml
                        .addOrUpdateRow("package", "published-at", coreBridgePackageId)
                        .updateRow("addresses", "wormhole", coreBridgePackageId);
                    break;
                case "token_bridge":
                    dependencyToml
                        .addOrUpdateRow("package", "published-at", tokenBridgePackageId)
                        .updateRow("addresses", "token_bridge", tokenBridgePackageId);
                    break;
                default:
                    throw new Error(`Unknown dependency ${dependencyPath}`);
            }
            fs_1.default.writeFileSync((0, publish_1.getDefaultTomlPath)(dependencyPath), dependencyToml.serialize(), "utf8");
        }
    }
};
const cleanupCoin = (packagePath) => {
    const paths = (0, publish_1.getAllLocalPackageDependencyPaths)((0, publish_1.getDefaultTomlPath)(packagePath));
    for (const dependencyPath of paths) {
        (0, publish_1.cleanupTempToml)(dependencyPath, false);
    }
};
