"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupMainToml = exports.cleanupTempToml = exports.publishPackage = exports.getPackageNameFromPath = exports.getDefaultTomlPath = exports.getAllLocalPackageDependencyPaths = exports.buildPackage = void 0;
const sui_js_1 = require("@mysten/sui.js");
const child_process_1 = require("child_process");
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
const MoveToml_1 = require("./MoveToml");
const utils_1 = require("./utils");
const buildPackage = (packagePath) => {
    if (!fs_1.default.existsSync(packagePath)) {
        throw new Error(`Package not found at ${packagePath}`);
    }
    return JSON.parse((0, child_process_1.execSync)(`sui move build --dump-bytecode-as-base64 --path ${packagePath} 2> /dev/null`, {
        encoding: "utf-8",
    }));
};
exports.buildPackage = buildPackage;
/**
 * Get Move.toml dependencies by looking for all lines of form 'local = ".*"'.
 * This works because network-specific Move.toml files should not contain
 * dev addresses, so the only lines that match this regex are the dependencies
 * that need to be replaced.
 * @param packagePath
 * @returns
 */
const getAllLocalPackageDependencyPaths = (tomlPath) => {
    const tomlStr = fs_1.default.readFileSync(tomlPath, "utf8").toString();
    const toml = new MoveToml_1.MoveToml(tomlStr);
    // Sanity check that Move.toml does not contain dev info since this breaks
    // building and publishing packages
    if (toml.getSectionNames().some((name) => name.includes("dev-dependencies")) ||
        toml.getSectionNames().some((name) => name.includes("dev-addresses"))) {
        throw new Error("Network-specific Move.toml should not contain dev-dependencies or dev-addresses.");
    }
    const packagePath = getPackagePathFromTomlPath(tomlPath);
    return [...tomlStr.matchAll(/local = "(.*)"/g)].map((match) => (0, path_1.resolve)(packagePath, match[1]));
};
exports.getAllLocalPackageDependencyPaths = getAllLocalPackageDependencyPaths;
const getDefaultTomlPath = (packagePath) => `${packagePath}/Move.toml`;
exports.getDefaultTomlPath = getDefaultTomlPath;
const getPackageNameFromPath = (packagePath) => packagePath.split("/").pop() || "";
exports.getPackageNameFromPath = getPackageNameFromPath;
const publishPackage = async (signer, network, packagePath) => {
    try {
        (0, exports.setupMainToml)(packagePath, network);
        const build = (0, exports.buildPackage)(packagePath);
        // Publish contracts
        const tx = new sui_js_1.TransactionBlock();
        if (network === "DEVNET") {
            // Avoid Error checking transaction input objects: GasBudgetTooHigh { gas_budget: 50000000000, max_budget: 10000000000 }
            tx.setGasBudget(10000000000);
        }
        const [upgradeCap] = tx.publish({
            modules: build.modules.map((m) => Array.from((0, sui_js_1.fromB64)(m))),
            dependencies: build.dependencies.map((d) => (0, sui_js_1.normalizeSuiObjectId)(d)),
        });
        // Transfer upgrade capability to deployer
        tx.transferObjects([upgradeCap], tx.pure(await signer.getAddress()));
        // Execute transactions
        const res = await (0, utils_1.executeTransactionBlock)(signer, tx);
        // Update network-specific Move.toml with package ID
        const publishEvents = (0, sui_js_1.getPublishedObjectChanges)(res);
        if (publishEvents.length !== 1) {
            throw new Error("No publish event found in transaction:" +
                JSON.stringify(res.objectChanges, null, 2));
        }
        updateNetworkToml(packagePath, network, publishEvents[0].packageId);
        // Return publish transaction info
        return res;
    }
    finally {
        (0, exports.cleanupTempToml)(packagePath);
    }
};
exports.publishPackage = publishPackage;
const cleanupTempToml = (packagePath, cleanupDependencies = true) => {
    const defaultTomlPath = (0, exports.getDefaultTomlPath)(packagePath);
    const tempTomlPath = getTempTomlPath(packagePath);
    if (fs_1.default.existsSync(tempTomlPath)) {
        // Clean up Move.toml for dependencies
        if (cleanupDependencies) {
            const dependencyPaths = (0, exports.getAllLocalPackageDependencyPaths)(defaultTomlPath);
            for (const path of dependencyPaths) {
                (0, exports.cleanupTempToml)(path);
            }
        }
        fs_1.default.renameSync(tempTomlPath, defaultTomlPath);
    }
};
exports.cleanupTempToml = cleanupTempToml;
const getPackagePathFromTomlPath = (tomlPath) => tomlPath.split("/").slice(0, -1).join("/");
const getTempTomlPath = (packagePath) => `${packagePath}/Move.temp.toml`;
const getTomlPathByNetwork = (packagePath, network) => `${packagePath}/Move.${network.toLowerCase()}.toml`;
const resetNetworkToml = (packagePath, network, recursive = false) => {
    const networkTomlPath = getTomlPathByNetwork(packagePath, network);
    const tomlStr = fs_1.default.readFileSync(networkTomlPath, "utf8").toString();
    const toml = new MoveToml_1.MoveToml(tomlStr);
    if (toml.isPublished()) {
        if (recursive) {
            const dependencyPaths = (0, exports.getAllLocalPackageDependencyPaths)(networkTomlPath);
            for (const path of dependencyPaths) {
                resetNetworkToml(path, network);
            }
        }
        const updatedTomlStr = toml
            .removeRow("package", "published-at")
            .updateRow("addresses", (0, exports.getPackageNameFromPath)(packagePath), "_")
            .serialize();
        fs_1.default.writeFileSync(networkTomlPath, updatedTomlStr, "utf8");
    }
};
const setupMainToml = (packagePath, network, checkDependencies = true, isDependency = false) => {
    const defaultTomlPath = (0, exports.getDefaultTomlPath)(packagePath);
    const tempTomlPath = getTempTomlPath(packagePath);
    const srcTomlPath = getTomlPathByNetwork(packagePath, network);
    if (fs_1.default.existsSync(tempTomlPath)) {
        // It's possible that this dependency has been set up by another package
        if (isDependency) {
            return;
        }
        throw new Error("Move.temp.toml exists, is there a publish in progress?");
    }
    // Make deploying on devnet more convenient by resetting Move.toml so we
    // don't have to manually reset them repeatedly during local development.
    // This is not recursive because we assume that packages are deployed bottom
    // up.
    if (!isDependency && network === "DEVNET") {
        resetNetworkToml(packagePath, network);
    }
    // Save default Move.toml
    if (!fs_1.default.existsSync(defaultTomlPath)) {
        throw new Error(`Invalid package layout. Move.toml not found at ${defaultTomlPath}`);
    }
    fs_1.default.renameSync(defaultTomlPath, tempTomlPath);
    // Set Move.toml from appropriate network
    if (!fs_1.default.existsSync(srcTomlPath)) {
        throw new Error(`Move.toml for ${network} not found at ${srcTomlPath}`);
    }
    fs_1.default.copyFileSync(srcTomlPath, defaultTomlPath);
    // Replace undefined addresses in base Move.toml
    const tomlStr = fs_1.default.readFileSync(defaultTomlPath, "utf8").toString();
    const toml = new MoveToml_1.MoveToml(tomlStr);
    const packageName = (0, exports.getPackageNameFromPath)(packagePath);
    if (!isDependency) {
        if (toml.isPublished()) {
            throw new Error(`Package ${packageName} is already published.`);
        }
        else {
            toml.updateRow("addresses", packageName, "0x0");
        }
        fs_1.default.writeFileSync(defaultTomlPath, toml.serialize());
    }
    else if (isDependency && !toml.isPublished()) {
        throw new Error(`Dependency ${packageName} is not published. Please publish it first.`);
    }
    // Set up Move.toml for dependencies
    if (checkDependencies) {
        const dependencyPaths = (0, exports.getAllLocalPackageDependencyPaths)(defaultTomlPath);
        for (const path of dependencyPaths) {
            (0, exports.setupMainToml)(path, network, checkDependencies, true);
        }
    }
};
exports.setupMainToml = setupMainToml;
const updateNetworkToml = (packagePath, network, packageId) => {
    const tomlPath = getTomlPathByNetwork(packagePath, network);
    const tomlStr = fs_1.default.readFileSync(tomlPath, "utf8");
    const updatedTomlStr = new MoveToml_1.MoveToml(tomlStr)
        .addRow("package", "published-at", packageId)
        .updateRow("addresses", (0, exports.getPackageNameFromPath)(packagePath), packageId)
        .serialize();
    fs_1.default.writeFileSync(tomlPath, updatedTomlStr, "utf8");
};
