"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerChain = exports.normalizeSuiType = exports.isValidSuiAddress = exports.isSuiPublishEvent = exports.isSuiCreateEvent = exports.isSameType = exports.getUpgradeCapObjectId = exports.getSigner = exports.getPublishedPackageId = exports.getProvider = exports.getPackageId = exports.getOwnedObjectId = exports.getCreatedObjects = exports.findOwnedObjectByType = exports.executeTransactionBlock = exports.assertSuccess = void 0;
const sui_js_1 = require("@mysten/sui.js");
const networks_1 = require("../networks");
const vaa_1 = require("../vaa");
const error_1 = require("./error");
const UPGRADE_CAP_TYPE = "0x2::package::UpgradeCap";
const assertSuccess = (res, error) => {
    var _a, _b;
    if (((_b = (_a = res === null || res === void 0 ? void 0 : res.effects) === null || _a === void 0 ? void 0 : _a.status) === null || _b === void 0 ? void 0 : _b.status) !== "success") {
        throw new Error(`${error} Response: ${JSON.stringify(res)}`);
    }
};
exports.assertSuccess = assertSuccess;
const executeTransactionBlock = async (signer, transactionBlock) => {
    // As of version 0.32.2, Sui SDK outputs a RPC validation warning when the
    // SDK falls behind the Sui version used by the RPC. We silence these
    // warnings since the SDK is often out of sync with the RPC.
    const consoleWarnTemp = console.warn;
    console.warn = () => { };
    // Let caller handle parsing and logging info
    const res = await signer.signAndExecuteTransactionBlock({
        transactionBlock,
        options: {
            showInput: true,
            showEffects: true,
            showEvents: true,
            showObjectChanges: true,
        },
    });
    console.warn = consoleWarnTemp;
    return res;
};
exports.executeTransactionBlock = executeTransactionBlock;
const findOwnedObjectByType = async (provider, owner, type, cursor) => {
    const res = await provider.getOwnedObjects({
        owner,
        filter: undefined,
        cursor: cursor || undefined,
        options: {
            showType: true,
        },
    });
    if (!res || !res.data) {
        throw new error_1.SuiRpcValidationError(res);
    }
    const object = res.data.find((d) => d.data.type === type);
    if (!object && res.hasNextPage) {
        return (0, exports.findOwnedObjectByType)(provider, owner, type, res.nextCursor);
    }
    else if (!object && !res.hasNextPage) {
        return null;
    }
    else {
        return object.data.objectId;
    }
};
exports.findOwnedObjectByType = findOwnedObjectByType;
const getCreatedObjects = (res) => {
    return res.objectChanges.filter(exports.isSuiCreateEvent).map((e) => ({
        type: e.objectType,
        objectId: e.objectId,
        owner: e.owner["AddressOwner"] || e.owner["ObjectOwner"] || e.owner,
    }));
};
exports.getCreatedObjects = getCreatedObjects;
const getOwnedObjectId = async (provider, owner, packageId, moduleName, structName) => {
    var _a;
    const type = `${packageId}::${moduleName}::${structName}`;
    // Upgrade caps are a special case
    if ((0, exports.normalizeSuiType)(type) === (0, exports.normalizeSuiType)(UPGRADE_CAP_TYPE)) {
        throw new Error("`getOwnedObjectId` should not be used to get the object ID of an `UpgradeCap`. Use `getUpgradeCapObjectId` instead.");
    }
    try {
        const res = await provider.getOwnedObjects({
            owner,
            filter: { StructType: type },
            options: {
                showContent: true,
            },
        });
        if (!res || !res.data) {
            throw new error_1.SuiRpcValidationError(res);
        }
        const objects = res.data.filter((o) => { var _a; return (_a = o.data) === null || _a === void 0 ? void 0 : _a.objectId; });
        if (objects.length === 1) {
            return (_a = objects[0].data) === null || _a === void 0 ? void 0 : _a.objectId;
        }
        else if (objects.length > 1) {
            const objectsStr = JSON.stringify(objects, null, 2);
            throw new Error(`Found multiple objects owned by ${owner} of type ${type}. This may mean that we've received an unexpected response from the Sui RPC and \`worm\` logic needs to be updated to handle this. Objects: ${objectsStr}`);
        }
        else {
            return null;
        }
    }
    catch (error) {
        // Handle 504 error by using findOwnedObjectByType method
        const is504HttpError = `${error}`.includes("504 Gateway Time-out");
        if (error && is504HttpError) {
            return (0, exports.findOwnedObjectByType)(provider, owner, type);
        }
        else {
            throw error;
        }
    }
};
exports.getOwnedObjectId = getOwnedObjectId;
// TODO(kp): remove this once it's in the sdk
async function getPackageId(provider, objectId) {
    var _a, _b, _c;
    let currentPackage;
    let nextCursor;
    do {
        const dynamicFields = await provider.getDynamicFields({
            parentId: objectId,
            cursor: nextCursor,
        });
        currentPackage = dynamicFields.data.find((field) => field.name.type.endsWith("CurrentPackage"));
        nextCursor = dynamicFields.hasNextPage ? dynamicFields.nextCursor : null;
    } while (nextCursor && !currentPackage);
    if (!currentPackage) {
        throw new Error("CurrentPackage not found");
    }
    const obj = await provider.getObject({
        id: currentPackage.objectId,
        options: {
            showContent: true,
        },
    });
    const packageId = ((_a = obj.data) === null || _a === void 0 ? void 0 : _a.content) && "fields" in obj.data.content
        ? (_c = (_b = obj.data.content.fields.value) === null || _b === void 0 ? void 0 : _b.fields) === null || _c === void 0 ? void 0 : _c.package
        : null;
    if (!packageId) {
        throw new Error("Unable to get current package");
    }
    return packageId;
}
exports.getPackageId = getPackageId;
const getProvider = (network, rpc) => {
    if (!network && !rpc) {
        throw new Error("Must provide network or RPC to initialize provider");
    }
    rpc = rpc || networks_1.NETWORKS[network]["sui"].rpc;
    if (!rpc) {
        throw new Error(`No default RPC found for Sui ${network}`);
    }
    return new sui_js_1.JsonRpcProvider(new sui_js_1.Connection({ fullnode: rpc }));
};
exports.getProvider = getProvider;
const getPublishedPackageId = (res) => {
    const publishEvents = (0, sui_js_1.getPublishedObjectChanges)(res);
    if (publishEvents.length !== 1) {
        throw new Error("Unexpected number of publish events found:" +
            JSON.stringify(publishEvents, null, 2));
    }
    return publishEvents[0].packageId;
};
exports.getPublishedPackageId = getPublishedPackageId;
const getSigner = (provider, network, customPrivateKey) => {
    const privateKey = customPrivateKey || networks_1.NETWORKS[network]["sui"].key;
    if (!privateKey) {
        throw new Error(`No private key found for Sui ${network}`);
    }
    const bytes = (0, sui_js_1.fromB64)(privateKey);
    const keypair = sui_js_1.Ed25519Keypair.fromSecretKey(bytes.slice(1));
    return new sui_js_1.RawSigner(keypair, provider);
};
exports.getSigner = getSigner;
/**
 * This function returns the object ID of the `UpgradeCap` that belongs to the
 * given package and owner if it exists.
 *
 * Structs created by the Sui framework such as `UpgradeCap`s all have the same
 * type (e.g. `0x2::package::UpgradeCap`) and have a special field, `package`,
 * we can use to differentiate them.
 * @param provider Sui RPC provider
 * @param owner Address of the current owner of the `UpgradeCap`
 * @param packageId ID of the package that the `UpgradeCap` was created for
 * @returns The object ID of the `UpgradeCap` if it exists, otherwise `null`
 */
const getUpgradeCapObjectId = async (provider, owner, packageId) => {
    var _a;
    const res = await provider.getOwnedObjects({
        owner,
        filter: { StructType: UPGRADE_CAP_TYPE },
        options: {
            showContent: true,
        },
    });
    if (!res || !res.data) {
        throw new error_1.SuiRpcValidationError(res);
    }
    const objects = res.data.filter((o) => {
        var _a, _b, _c, _d, _e, _f;
        return ((_a = o.data) === null || _a === void 0 ? void 0 : _a.objectId) &&
            ((_c = (_b = o.data) === null || _b === void 0 ? void 0 : _b.content) === null || _c === void 0 ? void 0 : _c.dataType) === "moveObject" &&
            ((_f = (_e = (_d = o.data) === null || _d === void 0 ? void 0 : _d.content) === null || _e === void 0 ? void 0 : _e.fields) === null || _f === void 0 ? void 0 : _f.package) === packageId;
    });
    if (objects.length === 1) {
        // We've found the object we're looking for
        return (_a = objects[0].data) === null || _a === void 0 ? void 0 : _a.objectId;
    }
    else if (objects.length > 1) {
        const objectsStr = JSON.stringify(objects, null, 2);
        throw new Error(`Found multiple upgrade capabilities owned by ${owner} from package ${packageId}. Objects: ${objectsStr}`);
    }
    else {
        return null;
    }
};
exports.getUpgradeCapObjectId = getUpgradeCapObjectId;
const isSameType = (a, b) => {
    try {
        return (0, exports.normalizeSuiType)(a) === (0, exports.normalizeSuiType)(b);
    }
    catch (e) {
        return false;
    }
};
exports.isSameType = isSameType;
const isSuiCreateEvent = (event) => {
    return event.type === "created";
};
exports.isSuiCreateEvent = isSuiCreateEvent;
const isSuiPublishEvent = (event) => {
    return event.type === "published";
};
exports.isSuiPublishEvent = isSuiPublishEvent;
const isValidSuiAddress = (objectId) => {
    return /^(0x)?[0-9a-f]{1,64}$/.test(objectId);
};
exports.isValidSuiAddress = isValidSuiAddress;
// todo(aki): this needs to correctly handle types such as
// 0x2::dynamic_field::Field<0x3c6d386861470e6f9cb35f3c91f69e6c1f1737bd5d217ca06a15f582e1dc1ce3::state::MigrationControl, bool>
const normalizeSuiType = (type) => {
    const tokens = type.split("::");
    if (tokens.length !== 3 || !(0, exports.isValidSuiAddress)(tokens[0])) {
        throw new Error(`Invalid Sui type: ${type}`);
    }
    return [(0, sui_js_1.normalizeSuiAddress)(tokens[0]), tokens[1], tokens[2]].join("::");
};
exports.normalizeSuiType = normalizeSuiType;
const registerChain = async (provider, network, vaa, coreBridgeStateObjectId, tokenBridgeStateObjectId, transactionBlock) => {
    if (network === "DEVNET") {
        // Modify the VAA to only have 1 guardian signature
        // TODO: remove this when we can deploy the devnet core contract
        // deterministically with multiple guardians in the initial guardian set
        // Currently the core contract is setup with only 1 guardian in the set
        const parsedVaa = (0, vaa_1.parse)(vaa);
        parsedVaa.signatures = [parsedVaa.signatures[0]];
        vaa = Buffer.from((0, vaa_1.serialiseVAA)(parsedVaa), "hex");
    }
    // Get package IDs
    const coreBridgePackageId = await getPackageId(provider, coreBridgeStateObjectId);
    const tokenBridgePackageId = await getPackageId(provider, tokenBridgeStateObjectId);
    // Register chain
    let tx = transactionBlock;
    if (!tx) {
        tx = new sui_js_1.TransactionBlock();
        tx.setGasBudget(1000000);
    }
    // Get VAA
    const [verifiedVaa] = tx.moveCall({
        target: `${coreBridgePackageId}::vaa::parse_and_verify`,
        arguments: [
            tx.object(coreBridgeStateObjectId),
            tx.pure([...vaa]),
            tx.object(sui_js_1.SUI_CLOCK_OBJECT_ID),
        ],
    });
    // Get decree ticket
    const [decreeTicket] = tx.moveCall({
        target: `${tokenBridgePackageId}::register_chain::authorize_governance`,
        arguments: [tx.object(tokenBridgeStateObjectId)],
    });
    // Get decree receipt
    const [decreeReceipt] = tx.moveCall({
        target: `${coreBridgePackageId}::governance_message::verify_vaa`,
        arguments: [tx.object(coreBridgeStateObjectId), verifiedVaa, decreeTicket],
        typeArguments: [
            `${tokenBridgePackageId}::register_chain::GovernanceWitness`,
        ],
    });
    // Register chain
    tx.moveCall({
        target: `${tokenBridgePackageId}::register_chain::register_chain`,
        arguments: [tx.object(tokenBridgeStateObjectId), decreeReceipt],
    });
    return tx;
};
exports.registerChain = registerChain;
